-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

-- Differential Electrics Helper
-- by wlkmanist, 2024

local M = {}
M.type = "auxiliary"

local abs = math.abs

local CMU = nil
local actuator = nil
local relevantDifferential = nil

-- Variables
local actuatorName = nil                -- "activeDiffLock"
local differentialName = nil            -- "differential_R"
local electricsActingName = nil         -- "differential_R_isActing"
local electricsLockingThreshold = 0.05


local function updateGFX()
  if actuator and electrics and electrics.values ~= nil then
    if relevantDifferential.mode == "activeLock" then
      electrics.values[electricsActingName] = (relevantDifferential.activeLockCoef >= electricsLockingThreshold) and 1 or 0
    elseif relevantDifferential.mode == "torqueVectoring" then
      electrics.values[electricsActingName] = (abs(relevantDifferential.diffTorqueSplitA - relevantDifferential.diffTorqueSplitB) >= electricsLockingThreshold / 2) and 1 or 0
    elseif actuatorName == "electronicDiffLock" or actuatorName == "electronicSplitShaftLock" and relevantDifferential.mode == "open" then
      electrics.values[electricsActingName] = actuator.isActing and 1 or 0
    else
      electrics.values[electricsActingName] = 0
    end
  end
end

local function init(jbeamData)
  log("D", "advanced_internals.diff_electrics_helper", "init")
  if jbeamData.actuatorName ~= nil then
    actuatorName = jbeamData.actuatorName
  end
  if jbeamData.electricsActingName ~= nil then
    electricsActingName = jbeamData.electricsActingName
  end
  if jbeamData.differentialName ~= nil then
    differentialName = jbeamData.differentialName
  end
  if jbeamData.electricsLockingThreshold ~= nil then
    electricsLockingThreshold = jbeamData.electricsLockingThreshold
  end
end

local function initSecondStage(jbeamData)
  log("D", "advanced_internals.diff_electrics_helper", "initSecondStage")
  if not CMU then
    log("W", "advanced_internals.diff_electrics_helper.initSecondStage", "No CMU present, disabling system...")
    shutdown()
    return
  end
  
  if not electricsActingName then
    log("E", "advanced_internals.diff_electrics_helper.initSecondStage", "No electricsActingName configured, disabling system...")
    return
  end

  if not differentialName then
    log("E", "advanced_internals.diff_electrics_helper.initSecondStage", "No differentialName configured, disabling system...")
    return
  end
  relevantDifferential = powertrain.getDevice(differentialName)

  if not relevantDifferential then
    log("E", "advanced_internals.diff_electrics_helper.initSecondStage", string.format("Can't find configured differential (%q), disabling system...", differentialName))
    return
  end

  if actuatorName ~= nil then
    actuator = CMU.getActuator(actuatorName)
    if actuator then
      M.updateGFX = updateGFX
      M.isActive = true
    else
      log("E", "advanced_internals.diff_electrics_helper.initSecondStage", "Failed getting actuator")
      return
    end
  else
    log("E", "advanced_internals.diff_electrics_helper.initSecondStage", "No actuatorName configured, disabling system...")
    return
  end
end

local function registerCMU(cmu)
  CMU = cmu
end


M.init = init
M.initSecondStage = initSecondStage
M.updateGFX = nop
M.registerCMU = registerCMU

return M
